(*----------------------------------------------------------------------------
  CDSThread.pas

  Description

    TCDSOpenThread is modified from an example at Dan Miser's web site (http:
    //www.distribucon.com), which is for Delphi 4 only, and this class is for
    Delphi 5 (MIDAS 3).
    You can use this class to open a client dataset in a thread, after all
    records are fetched, the client dataset's AfterOpen and AfterGetRecords
    events will be launched.

  Examples

    TCDSOpenThread.Create(ClientDataSet1);

  Notes

    - A thread will be freed automatically after it is terminated.
    - When using threads in a DLL, you must take care if there were threads
      still running when the DLL is being detached.

  History

    09-25-2000  Initial version.

  Written by Huanlin Tsai (huanlin_tsai@yahoo.com)
----------------------------------------------------------------------------*)
unit CDSThread;

interface

uses
  Windows, Messages, SysUtils, Classes, Db, DBClient;

type
  TCDSOpenThread = class(TThread)
    FCDS: TClientDataSet;
    FNewData: OleVariant;
    FHitEOF: Boolean;
    FCount: Integer;
    FStream: Pointer;
    FParams: OleVariant;
    FOwnerData: OleVariant;
  protected
    procedure Execute; override;
    procedure UpdateCDS;
    procedure OpenCDS;
  public
    constructor Create(CDS: TClientDataSet);
  end;

implementation

uses ActiveX, ComObj, StdVcl, Midas, Provider, DSIntf;

{ TCDSOpenThread }

constructor TCDSOpenThread.Create(CDS: TClientDataSet);
var
  AppServer: IAppServer;
begin
  FreeOnTerminate := True;
  FCDS := CDS;
  FCDS.Close;
  if FCDS.RemoteServer <> nil then
  begin
    FCount := FCDS.PacketRecords;
    FParams := PackageParams(FCDS.Params);
    // Get the app server;
    AppServer := CDS.RemoteServer.GetServer;
    // Marshall the AppServer interface into the stream...
    OleCheck(CoMarshalInterThreadInterfaceInStream(
      IAppServer, AppServer, IStream(FStream)));
  end;
  inherited Create(False);
end;

procedure TCDSOpenThread.Execute;
var
  AppServer: IAppServer;
  RecsOut: Integer;
  Options: TGetRecordOptions;
begin
  // Open CDS directly if RemoteServer not specified.
  if FCDS.RemoteServer = nil then
  begin
    Synchronize(OpenCDS);
    Exit;
  end;

  OleCheck(CoInitialize(nil));
  try
    //Get the marshalled interface
    OleCheck(CoGetInterfaceAndReleaseStream(
      IStream(FStream), IAppServer, AppServer));
    Options := [grMetaData];
    FHitEOF := False;
    while not FHitEOF do
    begin
      if Terminated then
        break;
    
      FNewData := AppServer.AS_GetRecords(
        FCDS.ProviderName,
        FCount,
        RecsOut,
        Byte(Options),      // Do NOT use byte(grMetaData) !!
        FCDS.CommandText,
        FParams,
        FOwnerData
      );
      FHitEOF := RecsOut <> FCount;
      // Update the CDS
      Synchronize(UpdateCDS);
    end;
    CoUninitialize;
  except
    CoUninitialize;
    raise
  end;
end;

procedure TCDSOpenThread.OpenCDS;
begin
  if Terminated then
    Exit;
  FCDS.Open;
end;

procedure TCDSOpenThread.UpdateCDS;
begin
  if Terminated then
    Exit;

  FCDS.AppendData(FNewData, FHitEOF);
  UnpackParams(FParams, FCDS.Params);

  if Assigned(FCDS.AfterGetRecords) then
    FCDS.AfterGetRecords(FCDS, FOwnerData);
end;


end.
